/**
 * 
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <time.h>
#include <json-c/json_object.h>
#include <json-c/json_tokener.h>

int8_t check_file_exists(const char *filename)
{
    FILE *file;
    file = fopen(filename, "r");
    if (file != NULL)
    {
        fclose(file);
        return 1;
    }

    return 0;
}

int main(int argc, char* argv[])
{
    time_t rawtime = time(NULL);
    struct tm *timeinfo;
    char dateString[32];
    char timeString[32];
    char filename[100];
    char jsonBuffer[1024];
    //char jsonStrOutput[4096];
    FILE *jsonFilePtr;

    struct json_object *parsed_json;
    struct json_object *latest_reading_object = json_object_new_object();
    struct json_object *temperature_object = json_object_new_object();
    //struct json_object *pressure_object = json_object_new_object();

    timeinfo = localtime(&rawtime);
    strftime(dateString, 32, "%F", timeinfo);
    strftime(timeString, 32, "%F_%H-%M-%S", timeinfo);

    snprintf(filename, sizeof(filename), "readings_%s.json", dateString);

    if (!check_file_exists(filename))
    {
        printf("File for today does not exist! Creating...\n");
        FILE *jsonFile = fopen(filename, "w");
        fputs("{}", jsonFile);
        fclose(jsonFile);
    }

    jsonFilePtr = fopen(filename, "r");
    if (jsonFilePtr == NULL)
    {
        printf("Failed to open %s for reading...\n", filename);
        exit(1);
    }
    fread(jsonBuffer, 1024, 1, jsonFilePtr);
    fclose(jsonFilePtr);
    printf("raw json read in: %s\n", jsonBuffer);

    parsed_json = json_tokener_parse(jsonBuffer);
    printf("json read in: %s\n", json_object_to_json_string_ext(parsed_json, JSON_C_TO_STRING_PRETTY));

    double fahrenheit = (23 * 1.8) + 32;
    double humidity = 100 - 80.82;

    json_object_object_add(temperature_object, "fahrenheit", json_object_new_double(fahrenheit));

    json_object_object_add(latest_reading_object, "temperature", temperature_object);
    json_object_object_add(latest_reading_object, "humidity_percent", json_object_new_double(humidity));

    json_object_object_add(parsed_json, timeString, latest_reading_object);

    printf("json to write out: %s\n", json_object_to_json_string_ext(latest_reading_object, JSON_C_TO_STRING_PRETTY));
    printf("new json to write out: %s\n", json_object_to_json_string_ext(parsed_json, JSON_C_TO_STRING_PRETTY));

    //uint32_t jsonStrSize = sizeof(json_object_to_json_string_ext(parsed_json, JSON_C_TO_STRING_PRETTY));
    //printf("jsonStrSize %i\n", jsonStrSize);
    //char jsonStrOutput[] = json_object_to_json_string_ext(parsed_json, JSON_C_TO_STRING_PRETTY);

    jsonFilePtr = fopen(filename, "w");
    if (jsonFilePtr == NULL)
    {
        printf("Failed to open %s for writing...\n", filename);
        exit(1);
    }
    fputs(json_object_to_json_string_ext(parsed_json, JSON_C_TO_STRING_PRETTY), jsonFilePtr);
    //fwrite(json_object_to_json_string_ext(parsed_json), 1, jsonStrSize, jsonFilePtr);
    fclose(jsonFilePtr);

    return 0;
}
